-- bascd vital
-- bascdvit.vhd


-- modle vital_compliant
library IEEE;
use IEEE.STD_LOGIC_1164.all;
library VITAL;
use VITAL.VITAL_Timing.all;


entity bascd is
   generic (tp : time := 3 ns ; -- retard
            tsu : time := 8 ns ; -- prpositionnement > 0
            tho : time := 2 ns ) ; -- maintien > 0
   port (   hor,d : in std_logic ;
            q : out std_logic ) ;
end bascd ;

-- entity declaration --
--entity P_FF is
--   generic(
--      InstancePath: STRING := "*";
--      tpd_C_Q                   :  DelayType01 := (0.000 ns, 0.000 ns);
--      tsetup_D_C_posedge        :  DelayTypeXX := 0.000 ns;
--      thold_C_D_posedge         :  DelayTypeXX := 0.000 ns;
--
--   port(
--      D                              :  in    STD_LOGIC;
--      C                              :  in    STD_LOGIC;
--      Q                              :  out   STD_LOGIC);
--end P_FF;

-- architecture body --

library VITAL;
use VITAL.VITAL_Primitives.all;

architecture VITAL_ARCH of bascd is
begin
   --------------------
   --  BEHAVIOR SECTION
   --------------------
   VITALBehavior : process (d, hor)

   -- timing check results
   VARIABLE Tviol_D_C : X01 := '0';
   VARIABLE Tmkr_D_C : TimeMarkerType;

   -- functionality results
   VARIABLE PrevData_Q : STD_LOGIC_VECTOR(1 to 5) := (others => 'X');
   VARIABLE Results : STD_LOGIC_VECTOR(1 to 1) := (others => 'X');
   ALIAS Q_zd : STD_LOGIC is Results(1);

   -- output glitch detection variables
   VARIABLE Q_GlitchData        : GlitchDataType;

---
   CONSTANT  L : VitalTableSymbolType := '0';
   CONSTANT  H : VitalTableSymbolType := '1';
   CONSTANT  X : VitalTableSymbolType := 'X';  --Unknown
   CONSTANT  NC : VitalTableSymbolType := 'S';-- no change
   CONSTANT  LH : VitalTableSymbolType := '/'; --rising edge
   CONSTANT  HL : VitalTableSymbolType := '\'; --falling edge
   CONSTANT  xH : VitalTableSymbolType := '^'; -- x to 1
   CONSTANT  Lx : VitalTableSymbolType := 'r'; -- 0 to x
   CONSTANT  xL : VitalTableSymbolType := 'v'; -- x to 0
   CONSTANT  Hx : VitalTableSymbolType := 'f'; -- 1 to x
   CONSTANT  Dc : VitalTableSymbolType := '-'; -- Don't care
   CONSTANT  D0 : VitalTableSymbolType := 'N'; -- any edge to 0
   CONSTANT  V : VitalTableSymbolType := 'X'; --  Violation 
   CONSTANT  E : VitalTableSymbolType := '*'; --  Any edge 
   CONSTANT  PV : VitalTableSymbolType := 'S'; --  previous value

   CONSTANT P_FF_tab : VitalStateTableType (1 to 10, 1 to 5) := (
--------------------------------
--Viol  D    C       Q   :   Q+
--------------------------------
   (V,  Dc,   Dc,     Dc,      PV),
   (Dc,  H,   LH,    Dc,      H ),
   (Dc,  L,   LH,    Dc,      L ),
   (Dc,  H,   xH,     H,      H ),
   (Dc,  L,   xH,     L,      L ),
   (Dc,  H,   Lx,     H,      H ),
   (Dc,  L,   Lx,     L,      L ),
   (Dc,  Dc,   D0,     Dc,      NC ),
   (Dc,  Dc,   Hx,     Dc,      NC ),
   (Dc,  E,   Dc,      Dc,      NC ) 
   );

   begin

      ------------------------
      --  Timing Check Section
      ------------------------
         VitalTimingCheck (d, "d", hor, "hor",
                           tsu, tsu,
                           tho, tho,
                           TRUE,
                           hor = '1',
                           "bascd",
                           Tmkr_D_C, Tviol_D_C);
      -------------------------
      --  Functionality Section
      -------------------------

      VitalStateTable(
        StateTable => P_FF_tab,
        DataIn => (Tviol_D_C, d, hor ),
        Result => Q_zd,
        PreviousDataIn => PrevData_Q);

      ----------------------
      --  Path Delay Section
      ----------------------
      VitalPropagatePathDelay (
       OutSignal => q,
       OutSignalName => "q",
       OutTemp => Q_zd,
       Paths => (0 => (hor'last_event, VitalExtendToFillDelay((tp,tp)), TRUE),
                 1 => (d'last_event, VitalExtendToFillDelay((tp,tp)), TRUE)
                 ),
       GlitchData => Q_GlitchData,
       GlitchMode => NoGlitch,
       GlitchKind => OnEvent);

end process;
end VITAL_ARCH;

