-- modle de bascule D avec specifications de timing fichier sdf
-- basd_mo2.vhd

library ieee,vital ;
use ieee.std_logic_1164.all ;
use ieee.VITAL_Timing.all;

entity bascd_sdf is
   generic (
            tpd_hor_q : VitalDelayType01 := (0 ns, 0 ns) ; -- retards
            tsetup_d_hor : time := 0 ns ; -- prpositionnement > 0
            thold_d_hor : time := 0 ns  ) ; -- maintien 
   port (   hor,d : in std_logic ;
            q : out std_logic ) ;
end bascd_sdf ;

library ieee ;
use ieee.VITAL_Primitives.all;
use ieee.VITAL_Timing.all;

architecture bascd_vital95 of bascd_sdf is
begin
   bascule : process (d, hor)
   -- donnes
   -- tests de timings
   VARIABLE viol_regles : X01 := '0';
   VARIABLE temps_d : VitalTimingDataType ;

   -- quations de fonctionnement : table de transitions
   VARIABLE anciens : STD_LOGIC_VECTOR(0 to 2) := (others => 'X');
   variable q_ideal : STD_LOGIC := 'X' ;

   CONSTANT transitions : VitalStateTableType (1 to 7, 1 to 5) := (
   --------------------------------
   --Viol  d   hor       q   :   q+
   --------------------------------
    ('X', '-', '-',     '-',    'X' ),-- erreur timing
    ('-', '1', '/',     '-',    '1' ),-- mise  1
    ('-', '0', '/',     '-',    '0' ),-- mise  0
    ('-', '-', '^',     '-',    'X' ),-- fronts d'horloges illgaux :
    ('-', '-', 'r',     '-',    'X' ),-- X->1 et O->X
    ('-', '-', 'F',     '-',    'S' ),-- fronts descendants
    ('-', '-', 'S',     '-',    'S' ) -- pas de front d'horloge
    );

   -- traitement de la sortie
   VARIABLE q_histoire : VitalGlitchDataType;
   begin
    -- contrle des rgles temporelles :
         VitalSetupHoldCheck (
                        TestSignal => d, 
                        TestSignalName => "d",
                        TestDelay => 0 ns,
                        RefSignal => hor,
                        RefSignalName => "hor",
                        RefDelay => 0 ns,
                        SetupHigh => tsetup_d_hor,
                        SetupLow => tsetup_d_hor,
                        HoldHigh => thold_d_hor,
                        HoldLow => thold_d_hor,
                        CheckEnabled => TRUE,
                        RefTransition => '/',
                        HeaderMsg => bascd_sdf'path_name,
                        XOn => TRUE,
                        MsgOn => TRUE,
                        MsgSeverity => warning,
                        TimingData => temps_d,
                        Violation => viol_regles);
    -- fonctionnement logique idal :
      VitalStateTable(
         StateTable => transitions,
         DataIn => (viol_regles, d, hor ),
         Result => q_ideal,
         PreviousDataIn => anciens);
    -- transaction avec retards :
      VitalPathDelay01 (
        OutSignal => q,
        OutSignalName => "q",
        OutTemp => q_ideal,
        Paths => (0 => (hor'last_event, tpd_hor_q, TRUE)),
        GlitchData => q_histoire);
end process;
end bascd_vital95 ;


