-- modle de bascule D avec specifications de timing
-- basd_mod.vhd

library ieee ;
use ieee.std_logic_1164.all ;

entity bascd is
   generic (tp : time := 3 ns ; -- retard
            tsu : time := 8 ns ; -- prpositionnement > 0
            tho : time := 2 ns ) ; -- maintien > 0
   port (   hor,d : in std_logic ;
            q : out std_logic ) ;
end bascd ;

architecture test of bascd is
begin
process
begin
   wait on hor,d ;
   if d'event and hor = '1' and not hor'stable(tho) then
      report bascd'path_name & " : violation de thold" ;
      q <= 'X' after tp ;
   elsif hor'event and hor'last_value = '0' and hor = '1' then
      if d'last_event < tsu then
         report bascd'path_name & " : violation de set up" ;
         q <= 'X' after tp ;
      else
         q <= d after tp ;
      end if ;
   elsif hor'event and (hor = '1' or hor'last_value = '0') then
       q <= 'X' after tp ;
      report bascd'path_name & " : transition d'horloge illgale" ;
   end if ;
end process ;
end test ;

use work.temporel.all ;

architecture generale of bascd is
begin
d_ff : process (hor,d)
   variable futurq, ancienq : std_logic ;
   variable timing_d_ok : boolean ;
   variable last_hor_front, last_d_event : time ;
   begin
    -- contrle des rgles temporelles
      test_regles(nom => bascd'path_name,
                  clk => hor,
                  entree => d,
                  clk_front => last_hor_front,
                  entree_change => last_d_event,
                  tsu => tsu,
                  th => tho,
                  ras => timing_d_ok);
    -- fonctionnement logique idal :
      if timing_d_ok then
         if rising_edge(hor) then
            futurq := d ;
         end if ;
      else
            futurq := 'X' ;
      end if ;
    -- transaction avec retards :
      propagat(sortie => q,
               nouvelle => futurq,
               ancienne => ancienq,
               tlh => tp, thl => tp) ;
end process d_ff ;
end generale ;

library VITAL;
use VITAL.VITAL_Primitives.all;
use VITAL.VITAL_Timing.all;

architecture vital_prim22b of bascd is
begin
   bascule : process (d, hor)
   -- donnes
   -- tests de timings
   VARIABLE viol_regles : X01 := '0';
   VARIABLE temps_d : TimeMarkerType := (-500 ns,NULL,NULL);

   -- quations de fonctionnement : table de transitions
   VARIABLE anciens : STD_LOGIC_VECTOR(0 to 2) := (others => 'X');
   variable q_ideal : STD_LOGIC := 'X' ;

   CONSTANT transitions : VitalStateTableType (1 to 7, 1 to 5) := (
   --------------------------------
   --Viol  d   hor       q   :   q+
   --------------------------------
    ('X', '-', '-',     '-',    'X' ),-- erreur timing
    ('-', '1', '/',     '-',    '1' ),-- mise  1
    ('-', '0', '/',     '-',    '0' ),-- mise  0
    ('-', '-', '^',     '-',    'X' ),-- fronts d'horloges illgaux :
    ('-', '-', 'r',     '-',    'X' ),-- X->1 et O->X
    ('-', '-', 'F',     '-',    'S' ),-- fronts descendants
    ('-', '-', 'S',     '-',    'S' ) -- pas de front d'horloge
    );

   -- traitement de la sortie
   VARIABLE q_histoire : GlitchDataType ;
   begin
    -- contrle des rgles temporelles :
         VitalTimingCheck (
                        TestPort => d, 
                        TestPortName => "d",
                        RefPort => hor,
                        RefPortName => "hor",
                        t_setup_hi => tsu,
                        t_setup_lo => tsu,
                        t_hold_hi => tho,
                        t_hold_lo => tho,
                        CheckEnabled => TRUE,
                        RefTransition => hor = '1',
                        HeaderMsg => bascd'path_name,
                        TimeMarker => temps_d,
                        Violation => viol_regles);
    -- fonctionnement logique idal :
      VitalStateTable(
         StateTable => transitions,
         DataIn => (viol_regles, d, hor ),
         Result => q_ideal,
         PreviousDataIn => anciens);
    -- transaction avec retards :
      VitalPropagatePathDelay (
        OutSignal => q,
        OutSignalName => "q",
        OutTemp => q_ideal,
        Paths => (0 => (hor'last_event, (tp,tp,tp,tp,tp,tp), TRUE)),
        GlitchData => q_histoire,
        GlitchMode => NoGlitch,
        GlitchKind => OnEvent);
end process;
end vital_prim22b ;

library ieee ;
use ieee.VITAL_Primitives.all;
use ieee.VITAL_Timing.all;

architecture bascd_vital95 of bascd is
begin
   bascule : process (d, hor)
   -- donnes
   -- tests de timings
   VARIABLE viol_regles : X01 := '0';
   VARIABLE temps_d : VitalTimingDataType ;

   -- quations de fonctionnement : table de transitions
   VARIABLE anciens : STD_LOGIC_VECTOR(0 to 2) := (others => 'X');
   variable q_ideal : STD_LOGIC := 'X' ;

   CONSTANT transitions : VitalStateTableType (1 to 7, 1 to 5) := (
   --------------------------------
   --Viol  d   hor       q   :   q+
   --------------------------------
    ('X', '-', '-',     '-',    'X' ),-- erreur timing
    ('-', '1', '/',     '-',    '1' ),-- mise  1
    ('-', '0', '/',     '-',    '0' ),-- mise  0
    ('-', '-', '^',     '-',    'X' ),-- fronts d'horloges illgaux :
    ('-', '-', 'r',     '-',    'X' ),-- X->1 et O->X
    ('-', '-', 'F',     '-',    'S' ),-- fronts descendants
    ('-', '-', 'S',     '-',    'S' ) -- pas de front d'horloge
    );

   -- traitement de la sortie
   VARIABLE q_histoire : VitalGlitchDataType;
   begin
    -- contrle des rgles temporelles :
         VitalSetupHoldCheck (
                        TestSignal => d, 
                        TestSignalName => "d",
                        TestDelay => 0 ns,
                        RefSignal => hor,
                        RefSignalName => "hor",
                        RefDelay => 0 ns,
                        SetupHigh => tsu,
                        SetupLow => tsu,
                        HoldHigh => tho,
                        HoldLow => tho,
                        CheckEnabled => TRUE,
                        RefTransition => '/',
                        HeaderMsg => bascd'path_name,
                        XOn => TRUE,
                        MsgOn => TRUE,
                        MsgSeverity => warning,
                        TimingData => temps_d,
                        Violation => viol_regles);
    -- fonctionnement logique idal :
      VitalStateTable(
         StateTable => transitions,
         DataIn => (viol_regles, d, hor ),
         Result => q_ideal,
         PreviousDataIn => anciens);
    -- transaction avec retards :
      VitalPathDelay (
        OutSignal => q,
        OutSignalName => "q",
        OutTemp => q_ideal,
        Paths => (0 => (hor'last_event, tp, TRUE)),
        GlitchData => q_histoire);
end process;
end bascd_vital95 ;

